@@ -11,6 +11,7 @@ from functools import partial
 from glob import glob
 import json
 import logging
+from numbers import Number
 import os
 import sys
 from time import sleep
@@ -26,13 +27,14 @@ from dask.compatibility import apply, unicode
 from dask.context import _globals
 from toolz import first, groupby, merge, valmap, keymap
 from tornado import gen
-from tornado.gen import Return, TimeoutError
+from tornado.gen import TimeoutError
 from tornado.locks import Event, Condition
 from tornado.ioloop import IOLoop, PeriodicCallback
 from tornado.queues import Queue
 
 from .batched import BatchedSend
 from .utils_comm import WrappedKey, unpack_remotedata, pack_data
+from .cfexecutor import ClientExecutor
 from .compatibility import Queue as pyQueue, Empty, isqueue
 from .core import connect, rpc, clean_exception, CommClosedError
 from .protocol import to_serialize
@@ -79,11 +81,11 @@ class Future(WrappedKey):
 
     def __init__(self, key, client):
         self.key = key
+        self._cleared = False
         tkey = tokey(key)
         self.client = client
         self.client._inc_ref(tkey)
         self._generation = self.client.generation
-        self._cleared = False
 
         if tkey in client.futures:
             self._state = client.futures[tkey]
@@ -106,9 +108,14 @@ class Future(WrappedKey):
         """ Is the computation complete? """
         return self.event.is_set()
 
-    def result(self):
-        """ Wait until computation completes. Gather result to local process """
-        result = sync(self.client.loop, self._result, raiseit=False)
+    def result(self, timeout=None):
+        """ Wait until computation completes. Gather result to local process.
+
+        If *timeout* seconds are elapsed before returning, a TimeoutError
+        is raised.
+        """
+        result = sync(self.client.loop,
+                      self._result, raiseit=False, callback_timeout=timeout)
         if self.status == 'error':
             six.reraise(*result)
         elif self.status == 'cancelled':
@@ -125,7 +132,7 @@ class Future(WrappedKey):
             if raiseit:
                 six.reraise(*exc)
             else:
-                raise Return(exc)
+                raise gen.Return(exc)
         elif self.status == 'cancelled':
             exception = CancelledError(self.key)
             if raiseit:
@@ -140,18 +147,22 @@ class Future(WrappedKey):
     def _exception(self):
         yield self.event.wait()
         if self.status == 'error':
-            raise Return(self._state.exception)
+            raise gen.Return(self._state.exception)
         else:
-            raise Return(None)
+            raise gen.Return(None)
 
-    def exception(self):
+    def exception(self, timeout=None):
         """ Return the exception of a failed task
 
+        If *timeout* seconds are elapsed before returning, a TimeoutError
+        is raised.
+
         See Also
         --------
         Future.traceback
         """
-        return sync(self.client.loop, self._exception)
+        return sync(self.client.loop,
+                    self._exception, callback_timeout=timeout)
 
     def add_done_callback(self, fn):
         """ Call callback on future when callback has finished
@@ -188,17 +199,20 @@ class Future(WrappedKey):
     def _traceback(self):
         yield self.event.wait()
         if self.status == 'error':
-            raise Return(self._state.traceback)
+            raise gen.Return(self._state.traceback)
         else:
-            raise Return(None)
+            raise gen.Return(None)
 
-    def traceback(self):
+    def traceback(self, timeout=None):
         """ Return the traceback of a failed task
 
         This returns a traceback object.  You can inspect this object using the
         ``traceback`` module.  Alternatively if you call ``future.result()``
         this traceback will accompany the raised exception.
 
+        If *timeout* seconds are elapsed before returning, a TimeoutError
+        is raised.
+
         Examples
         --------
         >>> import traceback  # doctest: +SKIP
@@ -210,7 +224,8 @@ class Future(WrappedKey):
         --------
         Future.exception
         """
-        return sync(self.client.loop, self._traceback)
+        return sync(self.client.loop,
+                    self._traceback, callback_timeout=timeout)
 
     @property
     def type(self):
@@ -345,8 +360,8 @@ class Client(object):
     --------
     distributed.scheduler.Scheduler: Internal scheduler
     """
-    def __init__(self, address=None, start=True, loop=None, timeout=3,
-                 set_as_default=True, scheduler_file=None):
+    def __init__(self, address=None, start=True, loop=None, timeout=5,
+                 set_as_default=True, scheduler_file=None, **kwargs):
         self.futures = dict()
         self.refcount = defaultdict(lambda: 0)
         self._should_close_loop = loop is None and start
@@ -358,6 +373,7 @@ class Client(object):
         self._pending_msg_buffer = []
         self.extensions = {}
         self.scheduler_file = scheduler_file
+        self._startup_kwargs = kwargs
 
         if hasattr(address, "scheduler_address"):
             # It's a LocalCluster or LocalCluster-compatible object
@@ -428,11 +444,18 @@ class Client(object):
             raise Exception("Client not running.  Status: %s" % self.status)
 
     @gen.coroutine
-    def _start(self, timeout=3, **kwargs):
+    def _start(self, timeout=5, **kwargs):
+        address = self._start_arg
         if self.cluster is not None:
             # Ensure the cluster is started (no-op if already running)
-            yield self.cluster._start()
-            self._start_arg = self.cluster.scheduler_address
+            try:
+                yield self.cluster._start()
+            except AttributeError:  # Some clusters don't have this method
+                pass
+            except Exception:
+                logger.info("Tried to start cluster and received an error. "
+                            "Proceeding.", exc_info=True)
+            address = self.cluster.scheduler_address
         elif self.scheduler_file is not None:
             while not os.path.exists(self.scheduler_file):
                 yield gen.sleep(0.01)
@@ -440,7 +463,7 @@ class Client(object):
                 try:
                     with open(self.scheduler_file) as f:
                         cfg = json.load(f)
-                    self._start_arg = cfg['address']
+                    address = cfg['address']
                     break
                 except (ValueError, KeyError):  # JSON file not yet flushed
                     yield gen.sleep(0.01)
@@ -450,14 +473,15 @@ class Client(object):
             from .deploy import LocalCluster
 
             try:
-                self.cluster = LocalCluster(loop=self.loop, start=False)
+                self.cluster = LocalCluster(loop=self.loop, start=False,
+                                            **self._startup_kwargs)
                 yield self.cluster._start()
             except (OSError, socket.error) as e:
                 if e.errno != errno.EADDRINUSE:
                     raise
                 # The default port was taken, use a random one
                 self.cluster = LocalCluster(scheduler_port=0, loop=self.loop,
-                                            start=False)
+                                            start=False, **self._startup_kwargs)
                 yield self.cluster._start()
 
             # Wait for all workers to be ready
@@ -465,17 +489,17 @@ class Client(object):
                    len(self.cluster.scheduler.ncores) < len(self.cluster.workers)):
                 yield gen.sleep(0.01)
 
-            self._start_arg = self.cluster.scheduler_address
+            address = self.cluster.scheduler_address
 
-        self.scheduler = rpc(self._start_arg, timeout=timeout)
+        self.scheduler = rpc(address, timeout=timeout)
         self.scheduler_comm = None
 
-        yield self.ensure_connected(timeout=timeout)
+        yield self._ensure_connected(timeout=timeout)
 
         self.coroutines.append(self._handle_report())
 
     @gen.coroutine
-    def reconnect(self, timeout=0.1):
+    def _reconnect(self, timeout=0.1):
         with log_errors():
             assert self.scheduler_comm.comm.closed()
             self.status = 'connecting'
@@ -487,13 +511,13 @@ class Client(object):
 
             while self.status == 'connecting':
                 try:
-                    yield self.ensure_connected()
+                    yield self._ensure_connected()
                     break
                 except EnvironmentError:
                     yield gen.sleep(timeout)
 
     @gen.coroutine
-    def ensure_connected(self, timeout=3):
+    def _ensure_connected(self, timeout=5):
         if self.scheduler_comm and not self.scheduler_comm.closed():
             return
 
@@ -526,6 +550,15 @@ class Client(object):
             self.start()
         return self
 
+    @gen.coroutine
+    def __aenter__(self):
+        yield self._start()
+        raise gen.Return(self)
+
+    @gen.coroutine
+    def __aexit__(self, typ, value, traceback):
+        yield self._shutdown()
+
     def __exit__(self, type, value, traceback):
         self.shutdown()
 
@@ -564,7 +597,7 @@ class Client(object):
                         logger.warn("Client report stream closed to scheduler")
                         logger.info("Reconnecting...")
                         self.status = 'connecting'
-                        yield self.reconnect()
+                        yield self._reconnect()
                         continue
                     else:
                         break
@@ -643,9 +676,12 @@ class Client(object):
         """ Send shutdown signal and wait until scheduler completes """
         with log_errors():
             if self.status == 'closed':
-                raise Return()
+                raise gen.Return()
             if self.status == 'running':
                 self._send_to_scheduler({'op': 'close-stream'})
+            if self._start_arg is None:
+                with ignoring(AttributeError):
+                    yield self.cluster._close()
             self.status = 'closed'
             if _global_client[0] is self:
                 _global_client[0] = None
@@ -674,12 +710,18 @@ class Client(object):
         # XXX handling of self.status here is not thread-safe
         if self.status == 'closed':
             return
+
+        self.status = 'closing'
+        if self._start_arg is None:
+            with ignoring(AttributeError):
+                self.cluster.close()
+
         sync(self.loop, self._shutdown, fast=True)
         assert self.status == 'closed'
 
         if self._should_close_loop:
             sync(self.loop, self.loop.stop)
-            self.loop.close(all_fds=True)
+            self.loop.close(all_fds=False)
             self._loop_thread.join(timeout=timeout)
         with ignoring(AttributeError):
             dask.set_options(get=self._previous_get)
@@ -687,8 +729,23 @@ class Client(object):
             dask.set_options(shuffle=self._previous_shuffle)
         if self.get == _globals.get('get'):
             del _globals['get']
-        with ignoring(AttributeError):
-            self.cluster.close()
+
+    def get_executor(self, **kwargs):
+        """ Return a concurrent.futures Executor for submitting tasks
+        on this Client.
+
+        Parameters
+        ----------
+        **kwargs:
+            Any submit()- or map()- compatible arguments, such as
+            `workers` or `resources`.
+
+        Returns
+        -------
+        An Executor object that's fully compatible with the concurrent.futures
+        API.
+        """
+        return ClientExecutor(self, **kwargs)
 
     def submit(self, func, *args, **kwargs):
         """ Submit a function application to the scheduler
@@ -1038,6 +1095,7 @@ class Client(object):
             d = yield self._scatter(keymap(tokey, data), workers, broadcast)
             raise gen.Return({k: d[tokey(k)] for k in data})
 
+        unpack = False
         if isinstance(data, dict):
             data2 = valmap(to_serialize, data)
             types = valmap(type, data)
@@ -1048,7 +1106,9 @@ class Client(object):
             data2 = list(map(to_serialize, data))
             types = list(map(type, data))
         else:
-            raise TypeError("Don't know how to scatter %s" % type(data))
+            data2 = [to_serialize(data)]
+            types = [type(data)]
+            unpack = True
         keys = yield self.scheduler.scatter(data=data2, workers=workers,
                                             client=self.id,
                                             broadcast=broadcast)
@@ -1059,8 +1119,7 @@ class Client(object):
         elif isinstance(data, (Iterable, Iterator)):
             out = [Future(k, self) for k in keys]
         else:
-            raise TypeError(
-                    "Input to scatter must be a list, iterator, or queue")
+            out = [Future(k, self) for k in keys]
 
         for key in keys:
             self.futures[key].finish(type=None)
@@ -1072,6 +1131,9 @@ class Client(object):
             for key in keys:
                 self.futures[key].type = types[key]
 
+        if unpack:
+            out = out[0]
+
         raise gen.Return(out)
 
     def _threaded_scatter(self, q_or_i, qout, **kwargs):
@@ -1105,7 +1167,7 @@ class Client(object):
 
         Parameters
         ----------
-        data: list, iterator, dict, or Queue
+        data: list, iterator, dict, Queue, or object
             Data to scatter out to workers.  Output type matches input type.
         workers: list of tuples (optional)
             Optionally constrain locations of data.
@@ -1123,6 +1185,9 @@ class Client(object):
         Examples
         --------
         >>> c = Client('127.0.0.1:8787')  # doctest: +SKIP
+        >>> c.scatter(1) # doctest: +SKIP
+        <Future: status: finished, key: c0a8a20f903a4915b94db8de3ea63195>
+
         >>> c.scatter([1, 2, 3])  # doctest: +SKIP
         [<Future: status: finished, key: c0a8a20f903a4915b94db8de3ea63195>,
          <Future: status: finished, key: 58e78e1b34eb49a68c65b54815d1b158>,
@@ -1281,7 +1346,7 @@ class Client(object):
 
         with temp_default_client(self):
             data = loads(out['data'])
-        raise Return(data)
+        raise gen.Return(data)
 
     def get_dataset(self, name):
         """
@@ -1302,7 +1367,7 @@ class Client(object):
         if response['status'] == 'error':
             six.reraise(*clean_exception(**response))
         else:
-            raise Return(response['result'])
+            raise gen.Return(response['result'])
 
     def run_on_scheduler(self, function, *args, **kwargs):
         """ Run a function on the scheduler process
@@ -1343,7 +1408,7 @@ class Client(object):
                 results[key] = resp['result']
             elif resp['status'] == 'error':
                 six.reraise(*clean_exception(**resp))
-        raise Return(results)
+        raise gen.Return(results)
 
     def run(self, function, *args, **kwargs):
         """
@@ -1391,7 +1456,7 @@ class Client(object):
                                                 wait=wait),
                                                 workers=workers)
         if not wait:
-            raise Return(None)
+            raise gen.Return(None)
         else:
             results = {}
             for key, resp in responses.items():
@@ -1399,7 +1464,7 @@ class Client(object):
                     results[key] = resp['result']
                 elif resp['status'] == 'error':
                     six.reraise(*clean_exception(**resp))
-            raise Return(results)
+            raise gen.Return(results)
 
     def run_coroutine(self, function, *args, **kwargs):
         """
@@ -2393,38 +2458,23 @@ class Client(object):
 Executor = Client
 
 
-class CompatibleExecutor(Client):
-    """ A concurrent.futures-compatible Client
-
-    A subclass of Client that conforms to concurrent.futures API,
-    allowing swapping in for other Clients.
-    """
-
-    def map(self, func, *iterables, **kwargs):
-        """ Map a function on a sequence of arguments
-
-        Returns
-        -------
-        iter_results: iterable
-            Iterable yielding results of the map.
-
-        See Also
-        --------
-        Client.map: for more info
-        """
-        list_of_futures = super(CompatibleExecutor, self).map(
-                                func, *iterables, **kwargs)
-        for f in list_of_futures:
-            yield f.result()
+def CompatibleExecutor(*args, **kwargs):
+    raise Exception("This has been moved to the Client.get_executor() method")
 
 
 @gen.coroutine
 def _wait(fs, timeout=None, return_when='ALL_COMPLETED'):
+    if timeout is not None and not isinstance(timeout, Number):
+        raise TypeError("timeout= keyword received a non-numeric value.\n"
+                "Beware that wait expects a list of values\n"
+                "  Bad:  wait(x, y, z)\n"
+                "  Good: wait([x, y, z])")
     fs = futures_of(fs)
-    if timeout is not None:
-        raise NotImplementedError("Timeouts not yet supported")
     if return_when == 'ALL_COMPLETED':
-        yield All({f.event.wait() for f in fs})
+        future = All({f.event.wait() for f in fs})
+        if timeout is not None:
+            future = gen.with_timeout(timedelta(seconds=timeout), future)
+        yield future
         done, not_done = set(fs), set()
         cancelled = [f.key for f in done
                      if f.status == 'cancelled']
@@ -2445,13 +2495,16 @@ def wait(fs, timeout=None, return_when='ALL_COMPLETED'):
     Parameters
     ----------
     fs: list of futures
+    timeout: number, optional
+        Time in seconds after which to raise a gen.TimeoutError
 
     Returns
     -------
     Named tuple of completed, not completed
     """
     client = default_client()
-    result = sync(client.loop, _wait, fs, timeout, return_when)
+    result = sync(client.loop, _wait, fs, timeout=timeout,
+                  return_when=return_when)
     return result
 
 
@@ -2518,8 +2571,17 @@ class AsCompleted(object):
     6
     12
     24
+
+    Optionally wait until the result has been gathered as well
+
+    >>> ac = as_completed([x, y, z], results=True)  # doctest: +SKIP
+    >>> for future, result in ac:  # doctest: +SKIP
+    ...     print(result)  # doctest: +SKIP
+    2
+    4
+    3
     """
-    def __init__(self, futures=None, loop=None):
+    def __init__(self, futures=None, loop=None, with_results=False):
         if futures is None:
             futures = []
         self.futures = defaultdict(lambda: 0)
@@ -2527,6 +2589,7 @@ class AsCompleted(object):
         self.lock = Lock()
         self.loop = loop or default_client().loop
         self.condition = Condition()
+        self.with_results = with_results
 
         if futures:
             for future in futures:
@@ -2535,11 +2598,16 @@ class AsCompleted(object):
     @gen.coroutine
     def track_future(self, future):
         yield _wait(future)
+        if self.with_results:
+            result = yield future._result()
         with self.lock:
             self.futures[future] -= 1
             if not self.futures[future]:
                 del self.futures[future]
-            self.queue.put_nowait(future)
+            if self.with_results:
+                self.queue.put_nowait((future, result))
+            else:
+                self.queue.put_nowait(future)
             self.condition.notify()
 
     def add(self, future):
@@ -2575,6 +2643,58 @@ class AsCompleted(object):
 
     next = __next__
 
+    def next_batch(self, block=True):
+        """ Get next batch of futures from as_completed iterator
+
+        Parameters
+        ----------
+        block: bool, optional
+            If True then wait until we have some result, otherwise return
+            immediately, even with an empty list.  Defaults to True.
+
+        Examples
+        --------
+        >>> ac = as_completed(futures)  # doctest: +SKIP
+        >>> client.gather(ac.next_batch())  # doctest: +SKIP
+        [4, 1, 3]
+
+        >>> client.gather(ac.next_batch(block=False))  # doctest: +SKIP
+        []
+
+        Returns
+        -------
+        List of futures or (future, result) tuples
+        """
+        if block:
+            batch = [next(self)]
+        else:
+            batch = []
+        while not self.queue.empty():
+            batch.append(self.queue.get())
+        return batch
+
+    def batches(self):
+        """
+        Yield all finished futures at once rather than one-by-one
+
+        This returns an iterator of lists of futures or lists of
+        (future, result) tuples rather than individual futures or individual
+        (future, result) tuples.  It will yield these as soon as possible
+        without waiting.
+
+        Examples
+        --------
+        >>> for batch in as_completed(futures).batches():  # doctest: +SKIP
+        ...     results = client.gather(batch)
+        ...     print(results)
+        [4, 2]
+        [1, 3, 7]
+        [5]
+        [6]
+        """
+        while True:
+            yield self.next_batch(block=True)
+
 
 as_completed = AsCompleted
 
