@@ -9,7 +9,7 @@ from tornado import gen
 from dask import do
 from distributed.client import _wait
 from distributed.diagnostics.progress_stream import (progress_quads,
-        nbytes_bar, progress_stream)
+        nbytes_bar, progress_stream, _incrementing_index_cache)
 from distributed.metrics import time
 from distributed.utils_test import inc, div, dec, gen_cluster
 from distributed.worker import dumps_task
@@ -20,6 +20,8 @@ def test_progress_quads():
            'erred': {'inc': 0, 'dec': 1, 'add': 0},
            'released': {'inc': 1, 'dec': 0, 'add': 1}}
 
+    _incrementing_index_cache.clear()
+
     d = progress_quads(msg, nrows=2)
     color = d.pop('color')
     assert len(set(color)) == 3
