@@ -4,8 +4,6 @@ import sys
 
 from dask.utils import Dispatch
 
-from .utils import ignoring
-
 try:  # PyPy does not support sys.getsizeof
     sys.getsizeof(1)
     getsizeof = sys.getsizeof
@@ -32,6 +30,7 @@ def sizeof_python_collection(seq):
 @sizeof.register_lazy("numpy")
 def register_numpy():
     import numpy as np
+
     @sizeof.register(np.ndarray)
     def sizeof_numpy_ndarray(x):
         return int(x.nbytes)
@@ -40,33 +39,46 @@ def register_numpy():
 @sizeof.register_lazy("pandas")
 def register_pandas():
     import pandas as pd
+    import numpy as np
+
+    def object_size(x):
+        if not len(x):
+            return 0
+        sample = np.random.choice(x, size=20, replace=True)
+        sample = list(map(sizeof, sample))
+        return sum(sample) / 20 * len(x)
+
     @sizeof.register(pd.DataFrame)
     def sizeof_pandas_dataframe(df):
-        p = int(df.memory_usage(index=True).sum())
-        obj = int((df.dtypes == object).sum() * len(df) * 100)
-        if df.index.dtype == object:
-            obj += len(df) * 100
-        return int(p + obj) + 1000
+        p = sum(c.memory_usage(index=False) for _, c in df.iteritems())
+        p += sizeof(df.index)
+        for col in df.columns:
+            col = df[col]
+            if col.dtype == object:
+                p += object_size(col._values)
+        return int(p) + 1000
 
     @sizeof.register(pd.Series)
     def sizeof_pandas_series(s):
         p = int(s.memory_usage(index=True))
         if s.dtype == object:
-            p += len(s) * 100
+            p += object_size(s._values)
         if s.index.dtype == object:
-            p += len(s) * 100
+            p += object_size(s.index)
         return int(p) + 1000
 
     @sizeof.register(pd.Index)
     def sizeof_pandas_index(i):
         p = int(i.memory_usage())
-        obj = len(i) * 100 if i.dtype == object else 0
-        return int(p + obj) + 1000
+        if i.dtype == object:
+            p += object_size(i)
+        return int(p) + 1000
 
 
 @sizeof.register_lazy("scipy")
 def register_spmatrix():
     from scipy import sparse
+
     @sizeof.register(sparse.dok_matrix)
     def sizeof_spmatrix_dok(s):
         return s.__sizeof__()
@@ -76,4 +88,3 @@ def register_spmatrix():
         return sum(
             sizeof(v) for v in s.__dict__.values()
         )
-
