@@ -1,8 +1,10 @@
 from __future__ import print_function, division, absolute_import
 
-from datetime import timedelta
+import random
 
+import dask
 from dask import delayed
+from time import sleep
 from tornado import gen
 
 from distributed import worker_client, Client, as_completed
@@ -67,7 +69,6 @@ def test_scatter_from_worker(c, s, a, b):
     start = time()
     while not all(v == 1 for v in s.ncores.values()):
         yield gen.sleep(0.1)
-        print(s.ncores)
         assert time() < start + 5
 
 
@@ -135,3 +136,49 @@ def test_async(c, s, a, b):
     while len(a.data) + len(b.data) > 1:
         yield gen.sleep(0.1)
         assert time() < start + 3
+
+
+@gen_cluster(client=True, ncores=[('127.0.0.1', 3)])
+def test_separate_thread_false(c, s, a):
+    a.count = 0
+    def f(i):
+        with worker_client(separate_thread=False) as client:
+            client.worker.count += 1
+            assert client.worker.count <= 3
+            sleep(random.random() / 40)
+            assert client.worker.count <= 3
+            client.worker.count -= 1
+        return i
+
+    futures = c.map(f, range(20))
+    results = yield c._gather(futures)
+    assert list(results) == list(range(20))
+
+
+@gen_cluster(client=True)
+def test_client_executor(c, s, a, b):
+    def mysum():
+        with worker_client() as c:
+            with c.get_executor() as e:
+                return sum(e.map(double, range(30)))
+
+    future = c.submit(mysum)
+    result = yield future._result()
+    assert result == 30 * 29
+
+
+def test_dont_override_default_get(loop):
+    import dask.bag as db
+    def f(x):
+        with worker_client() as c:
+            return True
+
+    b = db.from_sequence([1, 2])
+    b2 = b.map(f)
+
+    with Client(loop=loop, processes=False, set_as_default=True) as c:
+        assert dask.context._globals['get'] == c.get
+        for i in range(2):
+            b2.compute()
+
+        assert dask.context._globals['get'] == c.get
